;
;               Copyright (c)1995 Juha Pohto
;
;
; Filename   : SBLIB.ASM
; Author     : Juha Pohto
; Date       : 950621
; Description: USLIB-Emulator for Sound Blaster
; Version    : 1.11
;
; 1.00 - First working version
; 1.07 - New Mixing routine
; 1.08 - optimized mixing routine and some bugfixes
; 1.09 - Supports SB 1.0, SB 1.5, SB 2.0 and SB Pro
; 1.10 - Postprocessing routine added to mixer routine
;      - Mixingroutine optimized for 486 processors
; 1.11 - Added jmp $+2 after selfmodifying writes to make sure
;        that the date really is where it should be
;

ideal
locals
p386

sb_mixbuffsize  = 1024                          ; shold be enought (44100Hz/44Hz=1002)
sb_maxvoices    = 16

                align   4
sb_dramsize     dd      sb_dram                 ; samplememory size in bytes
sb_updaterate   dd      sb_rate                 ; sounddevice update rate in herz
sb_amplify      dd      sb_volamp               ; volume amplification

sb_type         db      ?       ; 0 = SB 1.0
                                ; 1 = SB 1.5
                                ; 2 = SB 2.0
                                ; 3 = SB Pro

sb_dmasize      dd      434

sb_base         dw      ?
sb_irq          dw      ?
sb_dma          dw      ?
sb_mixrate      dd      ?

sb_dmabase      db      87h,83h,81h,82h

                align   4
sb_oldpmirq     dd      ?
sb_oldrmirq     dd      ?
sb_oldirqmask   db      ?
sb_irqstubbuf   db      21 dup(?)

sb_envvar       db      "BLASTER",0

sb_speed        db      ?

                align   4
sb_mixingfreq   dd      ?
sb_mixrout      dd      ?

sb_mixroutines  dd      sb_mixmono
                dd      sb_mixmono
                dd      sb_mixmono
                dd      sb_mixstereo

sb_maxspeeds    db      210,210,233,233                 ; sb 1.0, sb 1.5, sb 2.0, sb pro
sb_muls         db      0,0,0,1

                align   4
sb_dmabuffer    dd      ?,?

sb_activevoices dd      0

sb_activevoice  dd      0

sb_start        dd      sb_maxvoices dup(0)             ; current address (22b.10b fp)
sb_loop         dd      sb_maxvoices dup(0)
sb_end          dd      sb_maxvoices dup(0)
sb_freq         dd      sb_maxvoices dup(0)
sb_adder        dd      sb_maxvoices dup(0)
sb_pos          dd      sb_maxvoices dup(0)
sb_volume       db      sb_maxvoices dup(0)
sb_flags        db      sb_maxvoices dup(3)
sb_panning      db      sb_maxvoices dup(0)

                align   4
sb_drambase     dd      ?
sb_dramtop      dd      ?
sb_dramaddress  dd      ?

sb_volumetable  dd      ?
sb_posttable    dd      ?

;͸͸
; Sb_Detect                                                                 
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: ax - errorcode: $00 - No Error                                        
;                      $01 - No SoundBlaster detected at current base port   
;                                                                            
; Description: Reset and detect Sound Blaster                                
;                                                                            
;ľ
;  
                align   4
sb_detect:      pushad

                mov     al,16
                call    sb_reset
                jc      @@no_blaster

                movzx   eax,[sb_type]
                call    [@@blasters+eax*4]

                popad
                sub     ax,ax
                clc
                ret

@@no_blaster:   popad
                mov     ax,02h
                stc
                ret

                align   4
@@blasters      dd      @@sb10
                dd      @@sb15
                dd      @@sb20
                dd      @@sbpro

                align   4
@@sb10:         ret
                align   4
@@sb15:         call    sb_getdspver
                cmp     ax,200h
                jl      @@sb10
                ret
                align   4
@@sb20:         call    sb_getdspver
                cmp     ax,201h
                jl      @@sb15
                ret
                align   4
@@sbpro:        call    sb_getdspver
                cmp     ax,300h
                jl      @@sb20
                sub     ax,ax                   ; reset sb pro mixer
                call    sb_setmixreg
                mov     ax,000ch
                call    sb_setmixreg
                mov     ax,220eh                ; set output to stereo
                call    sb_setmixreg
                mov     ax,0ff04h               ; set voice volume
                call    sb_setmixreg
                mov     ax,0ff22h               ; master volume
                call    sb_setmixreg
                mov     ax,26h                  ; turn other volumes to zero
                call    sb_setmixreg
                mov     ax,28h
                call    sb_setmixreg
                mov     ax,2eh
                call    sb_setmixreg
                mov     ax,0ah
                call    sb_setmixreg
                ret

;͸͸
; Sb_Initialize                                                             
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: CF=1 of there wasn't enought hi-memory free                           
;                                                                            
; Description: Initializes SoundBlaster                                      
;                                                                            
;ľ
;  
                align   4
sb_initialize:  pushad

                cli

                call    _himemsize
                mov     ebx,73728
                add     ebx,[sb_dramsize]
                cmp     eax,ebx
                jl      @@nomem

                mov     eax,[sb_dramsize]
                call    sb_allocdram

                mov     eax,[sb_amplify]
                call    sb_maketables

                mov     eax,[sb_mixrate]
                call    sb_setmixrate

                mov     eax,[sb_updaterate]
                call    sb_setupdaterate

                mov     bx,[sb_irq]
                call    _getirqmask
                mov     [sb_oldirqmask],al

                sub     al,al
                call    _setirqmask

                call    [_getirqvect]
                mov     [sb_oldpmirq],edx

                mov     edx,offset sb_interrupt
                call    [_setirqvect]

                mov     edi,offset sb_irqstubbuf
                call    _rmpmirqset
                mov     [sb_oldrmirq],eax

                ; allocate dmabuffers
                mov     edx,[_code32a]

                mov     eax,[_lomembase]
                add     eax,edx
                sub     ebx,ebx
                mov     bx,ax

                add     ebx,sb_mixbuffsize
                cmp     ebx,0ffffh
                jle     @@no_pagecross1
                sub     ax,ax
                add     eax,10000h
@@no_pagecross1:sub     eax,edx
                mov     [sb_dmabuffer],eax
                add     eax,sb_mixbuffsize
                mov     [_lomembase],eax

                add     eax,edx
                sub     ebx,ebx
                mov     bx,ax

                add     ebx,sb_mixbuffsize
                cmp     ebx,0ffffh
                jle     @@no_pagecross2
                sub     ax,ax
                add     eax,10000h
@@no_pagecross2:sub     eax,edx
                mov     [sb_dmabuffer+4],eax
                add     eax,sb_mixbuffsize
                mov     [_lomembase],eax

                mov     edi,[sb_dmabuffer]
                mov     esi,[sb_dmabuffer+4]
                mov     al,80h
                mov     ecx,sb_mixbuffsize
@@clear:        mov     [edi],al
                mov     [esi],al
                inc     edi
                inc     esi
                dec     ecx
                jnz     @@clear
                           
                call    sb_enableoutput

                mov     eax,[sb_dmabuffer]
                mov     ebx,1
                call    sb_startplaying

                sti

                clc
                popad
                ret

@@nomem:        stc
                popad
                ret

;͸͸
; Sb_GetDspVer                                                              
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: ax - Dsp version in HEX!                                              
;                                                                            
; Description: Read Sound Blaster DSP version                                
;                                                                            
;ľ
;  
                align   4
sb_getdspver:   push    dx
                mov     al,0e1h
                call    sb_writedsp
                mov     dx,[sb_base]
                add     dx,0eh
@@dataavail:    in      al,dx
                test    al,al
                jns     @@dataavail
                mov     dx,[sb_base]
                add     dx,0ah
                in      al,dx
                mov     ah,al
                in      al,dx
                pop     dx
                ret

;͸͸
; Sb_Reset                                                                  
;ľ                                            
;                                                                            
; In:  al - Active voices                                                    
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Reset SoundBlaster                                            
;                                                                            
;ľ
;  
                align   4
sb_reset:       push    ax ebx ecx edx

                and     eax,0ffh
                mov     [sb_activevoices],eax

                mov     bx,[sb_base]
                add     bx,06h

                mov     dx,bx
                mov     al,1
                out     dx,al

                mov     ecx,10
                mov     dx,300h
@@wait:         in      al,dx
                dec     ecx
                jnz     @@wait

                mov     dx,bx
                sub     al,al
                out     dx,al

                mov     ecx,1000
                sub     bx,06h
@@dataavail:    mov     dx,bx
                add     dx,0eh
                in      al,dx
                test    al,al
                jns     @@dataavail

                mov     dx,bx
                add     dx,0ah
                in      al,dx
                cmp     al,0aah
                je      @@sb_found
                dec     ecx
                jnz     @@dataavail

@@no_sb:        pop     edx ecx ebx ax
                stc
                ret

@@sb_found:     ; stop all voices
                mov     ebx,offset sb_flags
                mov     al,3
                mov     ecx,sb_maxvoices
@@stop_voice:   mov     [ebx],al
                inc     ebx
                dec     ecx
                jnz     @@stop_voice

                pop     edx ecx ebx ax
                clc
                ret

;͸͸
; Sb_SetMixReg                                                              
;ľ                                            
;                                                                            
; In:  al - Index                                                            
;      ah - Value                                                            
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Set SB Pro mixer register                                     
;                                                                            
;ľ
;  
                align   4
sb_setmixreg:   push    ax dx
                mov     dx,[sb_base]
                add     dx,4
                out     dx,al
                inc     dx
                mov     al,ah
                out     dx,al
                pop     dx ax
                ret

;͸͸
; Sb_WriteDsp                                                               
;ľ                                            
;                                                                            
; In:  al - Value to write to SoundBlaster Digital Signal Processor chip     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Writes value in al to SoundBlaster DSP chip                   
;                                                                            
;ľ
;  
                align   4
sb_writedsp:    push    ax dx

                mov     dx,[sb_base]
                add     dx,0ch

                mov     ah,al

@@wait:         in      al,dx
                test    al,80h
                jnz     @@wait

                mov     al,ah
                out     dx,al

                pop     dx ax
                ret


;͸͸
; Sb_EnableOuput                                                            
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Turns Speaker in SoundBlaster ON                              
;                                                                            
;ľ
;  
                align   4
sb_enableoutput:push    ax
                mov     al,0d1h
                call    sb_writedsp
                pop     ax
                ret


;͸͸
; Sb_DisableOuput                                                           
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Turns Speaker in SoundBlaster OFF                             
;                                                                            
;ľ
;  
                align   4
sb_disableoutput:
                push    ax
                mov     al,0d3h
                call    sb_writedsp
                pop     ax
                ret


;͸͸
; Sb_MakeTables                                                             
;ľ                                            
;                                                                            
; In:  Al - Amplifying                                                       
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Generate volume table and postprocessing table                
;                                                                            
;ľ
;  
                align   4
sb_maketables:  pushad

                push    ax

                mov     eax,256*65*4
                call    _gethimem
                mov     edi,eax
                mov     [sb_volumetable],eax

                mov     eax,2048*4
                call    _gethimem
                mov     [sb_posttable],eax

                sub     ebx,ebx
                mov     bl,al
                sub     cl,cl           ; volume
@@volumeloop:   sub     ch,ch           ; sample
                mov     esi,256
@@sampleloop:   sub     eax,eax
                mov     al,ch
                sub     eax,128
                movzx   ebx,cl
                imul    ebx
                sar     eax,6
                mov     [edi],eax
                add     edi,4
                inc     ch
                dec     esi
                jnz     @@sampleloop
                inc     cl
                cmp     cl,64
                jle     @@volumeloop

                pop     bx
                and     ebx,0ffh

                mov     eax,2048*16
                sub     edx,edx
                div     ebx             ; 2048*16/z = c = ax
                mov     ebx,2048
                sub     ebx,eax
                shr     ebx,1           ; (2048-c)/2 = a = bx
                mov     ecx,ebx
                add     ecx,eax         ; a+c=b=cx
                sub     ebp,ebp         ; x
                mov     edi,[sb_posttable]
@@loop:         mov     eax,ebp
                sub     eax,ebx         ;(x-A)
                shl     eax,8           ;*256
                mov     esi,ecx
                sub     esi,ebx         ;(B-A)
                sub     edx,edx
                idiv    esi             ;(x-A)*256/(B-A)
                cmp     eax,255
                jle     @@ok1
                mov     eax,255
@@ok1:          test    eax,eax
                jge     @@ok2
                sub     eax,eax
@@ok2:
                mov     [edi],eax
                add     edi,4
                inc     ebp
                cmp     ebp,2048
                jl      @@loop

                clc
                popad
                ret

@@memerror:     stc
                popad
                ret

;͸͸
; Sb_SetMixRate                                                             
;ľ                                            
;                                                                            
; In:  eax - Rate(Hz)                                                        
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Set software mixer mixrate                                    
;                                                                            
;ľ
;  
                align   4
sb_setmixrate:  push    eax ebx ecx edx

                movzx   ebx,[sb_type]
                mov     cl,[sb_muls+ebx]
                shl     eax,cl
                movzx   ecx,[sb_maxspeeds+ebx]

                mov     ebx,eax
                mov     eax,1000000
                sub     edx,edx
                div     ebx
                neg     eax
                add     eax,256
                cmp     ecx,eax
                jg      @@limit
                mov     eax,ecx
@@limit:        mov     [sb_speed],al
                neg     eax
                add     eax,256
                mov     ebx,eax
                mov     eax,1000000
                sub     edx,edx
                div     ebx

                movzx   ebx,[sb_type]
                mov     cl,[sb_muls+ebx]
                shr     eax,cl
                mov     [sb_mixingfreq],eax

                pop     edx ecx ebx eax
                ret

;͸͸
; Sb_UpdateRate                                                             
;ľ                                            
;                                                                            
; In:  eax - rate (herz)                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Set software mixer updaterate                                 
;                                                                            
;ľ
;  
                align   4
sb_setupdaterate:push    eax ebx edx

                mov     ebx,[sb_mixingfreq]
                xchg    eax,ebx
                sub     edx,edx
                div     ebx

                mov     [sb_dmasize],eax

                pop     edx ebx eax
                ret

;͸͸
; Sb_UnInitialize                                                           
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: UnInitializes SoundBlaster                                    
;                                                                            
;ľ
;  
                align   4
sb_uninitialize:pushad

                cli

                mov     bx,[sb_irq]
                mov     al,[sb_oldirqmask]
                call    _setirqmask

                mov     edx,[sb_oldpmirq]
                call    [_setirqvect]

                mov     eax,[sb_oldrmirq]
                call    _rmpmirqfree

                mov     [sb_irqon],0

                sti

                call    sb_disableoutput

                popad
                ret
  
;͸͸
; Sb_Interrupt                                                              
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: SoundBlaster Interrupt handler                                
;                                                                            
;ľ
;  
                align   4
sb_interrupt:   pushad

                cmp     [sb_irqon],1
                je      @@skipirq

                mov     [sb_irqon],1

                mov     al,20h
                out     20h,al

                mov     dx,[sb_base]
                add     dx,0eh
                in      al,dx

                mov     eax,[sb_dmabuffer]
                mov     ebx,[sb_dmasize]
                call    sb_startplaying

                mov     ebx,[sb_dmabuffer+4]
                mov     [sb_dmabuffer],ebx
                mov     [sb_dmabuffer+4],eax

                call    sb_mixer

                mov     [sb_irqon],0

                popad
                iretd

@@skipirq:      mov     al,20h
                out     20h,al

                mov     dx,[sb_base]
                add     dx,0eh
                in      al,dx

                popad
                iretd

sb_irqon        db      ?

;͸͸
; Sb_StartPlaying                                                           
;ľ                                            
;                                                                            
; In:  eax - address of sampledata (relative to _code32)                     
;      ebx - length of sampledata to play (65535 max!)                       
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Transfers samples to DSP chip using DMA                       
;                                                                            
;ľ
;  
                align   4
sb_startplaying:pushad
                mov     [@@address],eax
                mov     [@@length],ebx

                ; calculate page:offset

                add     eax,[_code32a]
                mov     [@@offset],ax
                shr     eax,16
                mov     [@@page],al

                ; set dma

                mov     ax,4
                add     ax,[sb_dma]
                out     0ah,al

                sub     al,al
                out     0ch,al

                mov     ax,48h
                add     ax,[sb_dma]
                out     0bh,al

                mov     bx,[@@offset]
                mov     al,bl
                mov     dx,[sb_dma]
                add     dx,dx
                out     dx,al
                mov     al,bh
                out     dx,al

                inc     dx

                mov     ebx,[@@length]
                dec     ebx
                mov     al,bl
                out     dx,al
                mov     al,bh
                out     dx,al

                sub     edx,edx
                mov     dx,[sb_dma]
                sub     dh,dh
                mov     dl,[edx+sb_dmabase]
                mov     al,[@@page]
                out     dx,al

                mov     al,1
                out     0ah,al

                ; set dsp

                mov     al,40h
                call    sb_writedsp
                mov     al,[sb_speed]
                call    sb_writedsp

                cmp     [sb_type],3
                je      @@sb_dmahispeed

                cmp     [sb_mixrate],22050
                ja      @@sb_dmahispeed
                jmp     @@sb_dmanormal


                align   4
@@address       dd      ?
@@length        dd      ?
@@offset        dw      ?
@@page          db      ?

                align   4
@@sb_dmanormal: mov     al,14h
                call    sb_writedsp
                mov     al,bl
                call    sb_writedsp
                mov     al,bh
                call    sb_writedsp
                popad
                ret

                align   4
@@sb_dmahispeed:mov     al,48h
                call    sb_writedsp
                mov     al,bl
                call    sb_writedsp
                mov     al,bh
                call    sb_writedsp
                mov     al,91h
                call    sb_writedsp
                popad
                ret

;͸͸
; Sb_Mixer                                                                  
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Handles mixing                                                
;                                                                            
;ľ
;  
                align   4
sb_mixer:       pushad

                mov     ebx,offset sb_mixbuffer
                mov     ecx,[sb_dmasize]
                sub     eax,eax
@@clearloop:    mov     [ebx],eax
                add     ebx,4
                dec     ecx
                jnz     @@clearloop

                mov     eax,[sb_activevoices]
                dec     eax
                mov     [@@voices],eax
@@mixchannels:  sub     eax,eax
                mov     al,[sb_type]
                mov     ebp,[@@voices]
                call    [sb_mixroutines+eax*4]
                dec     [@@voices]
                jge     @@mixchannels

                call    sb_updatedmabuffer

                popad
                ret

@@voices        dd      ?

;
; ebp - channel
;
                align   4
sb_mixmono:     test    [sb_flags+ebp],1
                jz      @@nostop
                mov     [sb_flags+ebp],3
                ret

                align   4
@@nostop:       mov     edi,offset sb_mixbuffer

                mov     eax,[sb_volumetable]
                mov     [dword @@voltable+3],eax
                mov     eax,[sb_adder+ebp*4]
                mov     [dword @@adder+2],eax
                mov     al,[sb_volume+ebp]
                mov     [@@volume],al
                mov     eax,[sb_end+ebp*4]
                mov     [dword @@end+2],eax
                mov     eax,[sb_loop+ebp*4]
                mov     [@@loop],eax
                mov     eax,[sb_dramaddress]
                mov     [dword @@dram+2],eax

                jmp     $+2                             ; make sure that selfmodifying code is updated

                movzx   edx,[sb_flags+ebp]
                mov     esi,[sb_pos+ebp*4]

                mov     ecx,[sb_dmasize]

                align   16
@@mixbuffer:    mov     ebx,esi
                shr     ebx,10
                sub     eax,eax
@@dram:         mov     al,[12345678h+ebx]
                mov     ah,[@@volume]
@@adder:        add     esi,12345678h
@@voltable:     mov     eax,[12345678h+eax*4]
                add     [edi],eax
                add     edi,4
@@end:          cmp     ebx,12345678h
                jg      @@smpend
                dec     ecx
                jnz     @@mixbuffer
                mov     [sb_pos+ebp*4],esi
                ret

                align   4
@@smpend:       test    edx,8
                jz      @@stop
                mov     eax,[@@loop]
                shl     eax,10
                mov     esi,eax
                dec     ecx
                jnz     @@mixbuffer
                mov     [sb_pos+ebp*4],esi
                ret

                align   4
@@stop:         or      [sb_flags+ebp],3
                ret

                align   4
@@loop          dd      ?
@@flags         db      ?
@@volume        db      ?

;
; ebp - channel
;
                align   4
sb_mixstereo:   test    [sb_flags+ebp],1
                jz      @@nostop
                mov     [sb_flags+ebp],3
                ret

                align   4
@@nostop:       mov     edi,offset sb_mixbuffer
                cmp     [sb_panning+ebp],7
                jl      @@left
                add     edi,4
@@left:
                mov     eax,[sb_volumetable]
                mov     [dword @@voltable+3],eax
                mov     eax,[sb_adder+ebp*4]
                mov     [dword @@adder+2],eax
                mov     al,[sb_volume+ebp]
                mov     [@@volume],al
                mov     eax,[sb_end+ebp*4]
                mov     [dword @@end+2],eax
                mov     eax,[sb_loop+ebp*4]
                mov     [@@loop],eax
                mov     eax,[sb_dramaddress]
                mov     [dword @@dram+2],eax

                jmp     $+2                             ; make sure that selfmodifying code is updated

                movzx   edx,[sb_flags+ebp]
                mov     esi,[sb_pos+ebp*4]

                mov     ecx,[sb_dmasize]
                shr     ecx,1

                align   16
@@mixbuffer:    mov     ebx,esi
                shr     ebx,10
                sub     eax,eax
@@dram:         mov     al,[12345678h+ebx]
                mov     ah,[@@volume]
@@adder:        add     esi,12345678h
@@voltable:     mov     eax,[12345678h+eax*4]
                add     [edi],eax
                add     edi,8
@@end:          cmp     ebx,12345678h
                jg      @@smpend
                dec     ecx
                jnz     @@mixbuffer
                mov     [sb_pos+ebp*4],esi
                ret

                align   4
@@smpend:       test    edx,8
                jz      @@stop
                mov     eax,[@@loop]
                shl     eax,10
                mov     esi,eax
                dec     ecx
                jnz     @@mixbuffer
                mov     [sb_pos+ebp*4],esi
                ret

                align   4
@@stop:         or      [sb_flags+ebp],3
                ret

                align   4
@@loop          dd      ?
@@flags         db      ?
@@volume        db      ?

                align   4
sb_updatedmabuffer:
                mov     esi,offset sb_mixbuffer
                mov     edi,[sb_dmabuffer]
                mov     ecx,[sb_dmasize]
                mov     edx,[sb_posttable]
                align   16
@@copy:         mov     eax,[esi]
                add     esi,4
                mov     eax,[edx+eax*4+4096]
                mov     [edi],al
                inc     edi
                dec     ecx
                jnz     @@copy
@@no_data:      ret

                align   16
sb_mixbuffer    dd      sb_mixbuffsize dup(?)

;͸͸
; Sb_PokeData                                                               
;ľ                                            
;                                                                            
; In:  al - value                                                            
;      edi - address                                                         
;                                                                            
; Out: -                                                                     
;                                                                            
; Description:                                                               
;                                                                            
;ľ
;  
                align   4
sb_pokedata:    push    edi
                add     edi,[sb_dramaddress]
                add     al,128
                mov     [edi],al
                pop     edi
                ret

;͸͸
; Sb_PeekData                                                               
;ľ                                            
;                                                                            
; In:  edi - address                                                         
;                                                                            
; Out: al - value                                                            
;                                                                            
; Description:                                                               
;                                                                            
;ľ
;  
                align   4
sb_peekdata:    push    edi
                add     edi,[sb_dramaddress]
                mov     al,[edi]
                sub     al,128
                pop     edi
                ret

;͸͸
; Sb_SelectVoice                                                            
;ľ                                            
;                                                                            
; In:  al - Voice                                                            
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Select active voice                                           
;                                                                            
;ľ
;  
                align   4
sb_selectvoice: push    eax
                dec     eax
                and     eax,0ffh
                mov     [sb_activevoice],eax
                pop     eax
                ret

;͸͸
; Sb_FrequencyControl                                                       
;ľ                                            
;                                                                            
; In:  eax - Frequency                                                       
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Set Frequency for active voice                                
;                                                                            
;ľ
;  
                align   4
sb_frequencycontrol:
                push    ebx ecx edx
                mov     ebx,[sb_activevoice]
                mov     [sb_freq+ebx*4],eax
                shl     eax,10
                sub     edx,edx
                mov     ecx,[sb_mixingfreq]
                div     ecx
                mov     [sb_adder+ebx*4],eax
                pop     edx ecx ebx
                ret

;͸͸
; Sb_CurrentAddress                                                         
;ľ                                            
;                                                                            
; In:  eax - address                                                         
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Sets voice current address                                    
;                                                                            
;ľ
;  
                align   4
sb_currentaddress:
                push    eax ebx
                mov     ebx,[sb_activevoice]
                mov     [sb_start+ebx*4],eax
                shl     eax,10
                mov     [sb_pos+ebx*4],eax
                pop     ebx eax
                ret

;͸͸
; Sb_StartingAddress                                                        
;ľ                                            
;                                                                            
; In:  eax - address                                                         
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Set voice starting address                                    
;                                                                            
;ľ
;  
                align   4
sb_startingaddress:
                push    ebx
                mov     ebx,[sb_activevoice]
                mov     [sb_loop+ebx*4],eax
                pop     ebx
                ret

;͸͸
; Sb_EndAddress                                                             
;ľ                                            
;                                                                            
; In:  eax - address                                                         
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Set voice endaddress                                          
;                                                                            
;ľ
;  
                align   4
sb_endaddress:  push    ebx
                mov     ebx,[sb_activevoice]
                mov     [sb_end+ebx*4],eax
                pop     ebx
                ret

;͸͸
; Sb_PanPosition                                                            
;ľ                                            
;                                                                            
; In:  al - panning                                                          
;                                                                            
; Out: -                                                                     
;                                                                            
; Description:                                                               
;                                                                            
;ľ
;  
                align   4
sb_panposition: push    ebx
                mov     ebx,[sb_activevoice]
                mov     [sb_panning+ebx],al
                pop     ebx
                ret

;͸͸
; Sb_CurrentVolume                                                          
;ľ                                            
;                                                                            
; In:  al - Volume                                                           
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Set current volume                                            
;                                                                            
;ľ
;  
                align   4
sb_currentvolume:
                push    ebx
                mov     ebx,[sb_activevoice]
                mov     [sb_volume+ebx],al
                pop     ebx
                ret

;͸͸
; Sb_WaitVolRamp                                                            
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Does absolutely nothing                                       
;                                                                            
;ľ
;  
                align   4
sb_waitvolramp: ret

;͸͸
; Sb_VoiceControl                                                           
;ľ                                            
;                                                                            
; In:  al - Controlbyte                                                      
;                                                                            
; Out: -                                                                     
;                                                                            
; Description:                                                               
;               BIT   DESCRIPTION                                            
;               ---   ------------------------------------------------       
;                0     Voice Stopped                                         
;                1     Stop Voice                                            
;                2     -                                                     
;                3     Loop enable                                           
;                4     -                                                     
;                5     -                                                     
;                6     -                                                     
;                7     -                                                     
;                                                                            
;ľ
;  
                align   4
sb_voicecontrol:push    ebx
                mov     ebx,[sb_activevoice]
                test    al,2
                jz      @@no_stopvoice
                and     al,0feh
@@no_stopvoice: mov     [sb_flags+ebx],al
                pop     ebx
                ret

;͸͸
; Sb_AllocDRAM                                                              
;ľ                                            
;                                                                            
; In:  eax - DRAM size                                                       
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Allocates virtual DRAM from high memory                       
;                                                                            
;ľ
;  
                align   4
sb_allocdram:   push    eax
                mov     [sb_drambase],0
                mov     [sb_dramtop],eax
                call    _gethimem
                jc      @@no_mem
                mov     [sb_dramaddress],eax
                pop     eax
                ret
@@no_mem:       pop     eax
                sub     eax,eax
                ret


;͸͸
; Sb_GetMem                                                                 
;ľ                                            
;                                                                            
; In:  eax - size                                                            
;                                                                            
; Out: cf=1 => Not enought memory                                            
;      eax - address of memoryblock                                          
;                                                                            
; Description: Allocate Virtual DRAM                                         
;                                                                            
;ľ
;  
                align   4
sb_getmem:      push    ebx
                mov     ebx,[sb_drambase]
                add     ebx,eax
                cmp     ebx,[sb_dramtop]
                jle     @@allocok
                pop     ebx
                stc
                ret
@@allocok:      mov     ebx,[sb_drambase]
                add     [sb_drambase],eax
                mov     eax,ebx
                pop     ebx
                clc
                ret
