;
;               Copyright (c)1995 Juha Pohto
;
;
; Filename   : USLIB101.ASM
; Author     : Juha Pohto
; Date       : 950611
; Description: Gravis Ultrasound library
; Version    : 1.02
;
; 1.01 - Fixed for Slender's Sound System configuration menu
; 1.02 - Volume rampping added
;

ideal
locals
p386

                align 4
us_mixctrl      dw      ?       ; (2x0h)
us_irqstat      dw      ?       ; (2x6h)
us_timerctrl    dw      ?       ; (2x8h)
us_timerdata    dw      ?       ; (2x9h)
us_irqctrl      dw      ?       ; (2xbh)
us_midictrl     dw      ?       ; (3x0h)
us_mididata     dw      ?       ; (3x1h)
us_page         dw      ?       ; (3x2h)
us_regselect    dw      ?       ; (3x3h)
us_datalow      dw      ?       ; (3x4h)
us_datahigh     dw      ?       ; (3x5h)
us_dram         dw      ?       ; (3x7h)

us_2x0          db      ?
us_2x9          db      ?
us_revisionid   db      ?       ; id      board revision
                                ; 0xff    -pre 3.7 boards.
                                ;         -ics mixer not present.
                                ; 5       -rev 3.7 with ics mixer.
                                ;          some r/l: flip problems.
                                ; 6-9     -revision 3.7 and above.
                                ;          ics mixer present
                                ; 0x0a-nn -ultramax.
                                ;          cs4231 present, no ics mixer

                align 4
us_base         dw      ?       ; base port
us_irq          dw      ?       ; gf1 irq               ; not used!
us_dma          dw      ?       ; dma channel           ; not used!

                align 4
us_drambase     dd      ?                 ; base of memory
us_dramtop      dd      ?                 ; top of memory (memory size)

us_envvar       db      "ULTRASND",0

                align 4
us_divisor      dw      ?                 ; divisor for current number of voices

us_divisors     dw      44100,41160,38587,36317,34300,32494,30870,29400
                dw      28063,26843,25725,24696,23746,22866,22050,21289
                dw      20580,19916,19293

us_volumetable  dw      0150h,08f1h,09f1h,0ab5h,0af1h,0b97h,0bb5h,0bd3h
                dw      0bf1h,0c88h,0c97h,0ca6h,0cb5h,0cc4h,0cd3h,0ce2h
                dw      0cf1h,0d80h,0d88h,0d8fh,0d97h,0d9eh,0da6h,0dadh
                dw      0db5h,0dbch,0dc4h,0dcbh,0dd3h,0ddah,0de2h,0de9h
                dw      0df1h,0df8h,0e80h,0e84h,0e88h,0e8bh,0e8fh,0e93h
                dw      0e97h,0e9ah,0e9eh,0ea2h,0ea6h,0ea9h,0eadh,0eb1h
                dw      0eb5h,0eb8h,0ebch,0ec0h,0ec4h,0ec7h,0ecbh,0ecfh
                dw      0ed3h,0ed6h,0edah,0edeh,0ee2h,0ee5h,0ee9h,0eedh
                dw      0ef0h


;͸͸
; Us_Detect                                                                         
;ľ                                                    
;                                                                                    
; In:  -                                                                             
;                                                                                    
; Out: ax - errorcode: $00 No Error                                                  
;                      $01 No ULTRASND environment variable defined                  
;                      $02 No UltraSound detected                                    
;                                                                                    
; Description: Detect UltraSound                                                     
;                                                                                    
;ľ
;  
                align   4
us_detect:      pushad

                call    us_setinterface

                mov     al,16
                call    us_reset
                jc      @@no_ultrasound

                popad
                sub     ax,ax
                clc
                ret

@@no_env:       popad
                mov     ax,01h
                stc
                ret

@@no_ultrasound:popad
                mov     ax,02h
                stc
                ret


;͸͸
; Us_Initialize                                                                     
;ľ                                                    
;                                                                                    
; In:  -                                                                             
;                                                                                    
; Out: -                                                                             
;                                                                                    
; Description:                                                                       
;                                                                                    
;ľ
;  
                align   4
us_initialize:  pushad

                call    us_getdramsize

                call    us_enableoutput

                popad
                ret


;͸͸
; Us_UnInitialize                                                                   
;ľ                                                    
;                                                                                    
; In:  -                                                                             
;                                                                                    
; Out: -                                                                             
;                                                                                    
; Description: Uninitialize UltraSound                                               
;                                                                                    
;ľ
;  
                align   4
us_uninitialize:pushad

                call    us_disableoutput

                mov     al,16
                call    us_reset

                mov     ecx,31
@@voiceloop:    mov     al,cl
                call    us_selectvoice
                mov     dx,[us_regselect]
                mov     al,00h
                out     dx,al
                add     dx,2
                mov     al,03h
                out     dx,al
                mov     al,0dh
                sub     dx,2
                out     dx,al
                add     dx,2
                mov     al,03h
                out     dx,al
                call    us_delay
                sub     eax,eax
                call    us_frequencycontrol
                call    us_currentaddress
                call    us_startingaddress
                call    us_endaddress
                mov     al,3
                call    us_voicecontrol
                dec     ecx
                jge     @@voiceloop

                popad
                ret

;͸͸
; Us_UdateRate                                                                      
;ľ                                                    
;                                                                                    
; In:  -                                                                             
;                                                                                    
; Out: -                                                                             
;                                                                                    
; Description:                                                                       
;                                                                                    
;ľ
;  
                align   4
us_updaterate:  ret

;͸͸
; Us_Interface                                                              
;ľ                                                    
;                                                                            
; In:  -                                                                     
; Out: -                                                                     
;                                                                            
; Description: Set interface                                                 
;                                                                            
;ľ
;  
                align   4
us_setinterface:push    eax ebx
                sub     eax,eax
                mov     ax,[us_base]
                mov     [us_mixctrl],ax
                lea     ebx,[eax+006h]
                mov     [us_irqstat],bx
                lea     ebx,[eax+008h]
                mov     [us_timerctrl],bx
                lea     ebx,[eax+009h]
                mov     [us_timerdata],bx
                lea     ebx,[eax+00bh]
                mov     [us_irqctrl],bx
                lea     ebx,[eax+100h]
                mov     [us_midictrl],bx
                lea     ebx,[eax+101h]
                mov     [us_mididata],bx
                lea     ebx,[eax+102h]
                mov     [us_page],bx
                lea     ebx,[eax+103h]
                mov     [us_regselect],bx
                lea     ebx,[eax+104h]
                mov     [us_datalow],bx
                lea     ebx,[eax+105h]
                mov     [us_datahigh],bx
                lea     ebx,[eax+107h]
                mov     [us_dram],bx
                pop     ebx eax
                ret

;͸͸
; Us_GetDRAMSize                                                            
;ľ                                                    
;                                                                            
; In:  -                                                                     
; Out: -                                                                     
;                                                                            
; Description: Get amount of memory on card                                  
;                                                                            
;ľ
;  
                align   4
us_getdramsize: push    esi edi ax
                ; test  256
                mov     edi,40000h
                mov     al,55h
                call    us_pokedata
                mov     esi,40000h
                call    us_peekdata
                cmp     al,55h
                jne     @@done
                ; test 512
                mov     edi,80000h
                call    us_pokedata
                mov     esi,80000h
                call    us_peekdata
                cmp     al,55h
                jne     @@done
                ; test 768
                mov     edi,0c0000h
                call    us_pokedata
                mov     esi,0c0000h
                call    us_peekdata
                cmp     al,55h
                jne     @@done
                ; 1024
                mov     edi,100000h
@@done:         mov     [us_dramtop],edi
                mov     [us_drambase],0
                pop     ax edi esi
                ret

;͸͸
; Us_GetMem                                                                 
;ľ                                                    
;                                                                            
; In:  EAx=Size                                                              
; Out: If CF=1 then not enought memory                                       
;      EAx=DRam address or ?                                                 
;                                                                            
;                                                                            
; Description: Allocate DRAM                                                 
;                                                                            
;ľ
;  
                align   4
us_getmem:      push    ecx ebx
                mov     ebx,[us_drambase]
                lea     ecx,[eax+ebx]
                mov     eax,ebx
                cmp     ecx,[us_dramtop]
                jg      @@nomem
                mov     [us_drambase],ecx
                pop     ebx ecx
                clc
                ret
@@nomem:        pop     ebx ecx
                stc
                ret

;͸͸
; Us_Delay                                                                  
;ľ                                                    
;                                                                            
; In:  -                                                                     
; Out: -                                                                     
;                                                                            
; Description: Delay for 4.8ms                                               
;                                                                            
;ľ
;  
                align   4
us_delay:       push    dx ax
                mov     dx,300h
                in      al,dx
                in      al,dx
                in      al,dx
                in      al,dx
                in      al,dx
                in      al,dx
                in      al,dx
                pop     ax dx
                ret

;͸͸
; Us_PokeData                                                               
;ľ                                                    
;                                                                            
; In:  Al=Data                                                               
;      EDi=Address                                                           
; Out: -                                                                     
;                                                                            
; Description: Poke data into DRAM                                           
;                                                                            
;ľ
;  
                align   4
us_pokedata:    push    dx edi ax
                mov     dx,[us_regselect]
                mov     al,43h
                out     dx,al
                inc     dx
                mov     ax,di
                out     dx,ax
                dec     dx
                mov     al,44h
                out     dx,al
                add     dx,2
                shr     edi,16
                mov     ax,di
                out     dx,al
                add     dx,2
                pop     ax
                out     dx,al
                pop     edi dx
                ret

;͸͸
; Us_PeekData                                                               
;ľ                                                    
;                                                                            
; In:  ESi=Address                                                           
; Out: Al=Data                                                               
;                                                                            
; Description: Peek data from DRAM                                           
;                                                                            
;ľ
;  
                align   4
us_peekdata:    push    dx esi ax
                mov     dx,[us_regselect]
                mov     al,43h
                out     dx,al
                inc     dx
                mov     ax,si
                out     dx,ax
                dec     dx
                mov     al,44h
                out     dx,al
                add     dx,2
                shr     esi,16
                mov     ax,si
                out     dx,al
                add     dx,2
                pop     ax
                in      al,dx
                pop     esi dx
                ret


;͸͸
; Us_Reset                                                                  
;ľ                                                    
;                                                                            
; In:  Al=Number of active voices (14..32)                                   
; Out: If CF=1 then Ax=Errorcode: $01 Bad number of voices                   
;                                 $02 No UltraSound detected                 
;                                                                            
; Description: Reset and probe ultrasound                                    
;                                                                            
;ľ
;  
                align   4
us_reset:       cli
                cmp     al,14
                jl      @@voices
                cmp     al,32
                jg      @@voices
                call    us_probe
                jc      @@no_ultrasound
                pushad
                push    ax
                call    us_disableoutput
                mov     al,[us_2x0]
                or      al,08h
                mov     [us_2x0],al
                mov     dx,[us_mixctrl]
                out     dx,al
                sub     al,al
                sub     edi,edi
                call    us_pokedata
                inc     edi
                call    us_pokedata
                mov     al,4ch
                mov     dx,[us_regselect]
                out     dx,al
                sub     al,al
                add     dx,2
                out     dx,al
                mov     ecx,12
@@delayloop1:   call    us_delay
                dec     ecx
                jnz     @@delayloop1
                mov     al,4ch
                sub     dx,2
                out     dx,al
                mov     al,01h
                add     dx,2
                out     dx,al
                mov     al,03h
                sub     dx,5
                out     dx,al
                mov     ecx,12
@@delayloop2:   call    us_delay
                dec     ecx
                jnz     @@delayloop2
                sub     al,al
                out     dx,al
                mov     al,41h
                add     dx,3
                out     dx,al
                sub     al,al
                add     dx,2
                out     dx,al
                mov     al,45h
                sub     dx,2
                out     dx,al
                sub     al,al
                add     dx,2
                out     dx,al
                mov     al,49h
                sub     dx,2
                out     dx,al
                sub     al,al
                add     dx,2
                out     dx,al
                pop     bx
                and     ebx,0ffh
                mov     ax,[us_divisors-28+ebx*2]
                mov     [us_divisor],ax
                dec     bl
                or      bl,0c0h
                mov     al,0eh
                sub     dx,2
                out     dx,al
                add     dx,2
                mov     al,bl
                out     dx,al
                mov     dx,[us_irqstat]
                in      al,dx
                mov     dx,[us_regselect]
                mov     al,49h
                out     dx,al
                add     dx,2
                in      al,dx
                mov     al,41h
                sub     dx,2
                out     dx,al
                add     dx,2
                in      al,dx
                mov     ecx,31
@@voiceloop:    mov     al,cl
                call    us_selectvoice
                mov     dx,[us_regselect]
                mov     al,00h
                out     dx,al
                add     dx,2
                mov     al,03h
                out     dx,al
                mov     al,0dh
                sub     dx,2
                out     dx,al
                add     dx,2
                mov     al,03h
                out     dx,al
                call    us_delay
                sub     eax,eax
                call    us_frequencycontrol
                call    us_currentaddress
                call    us_startingaddress
                call    us_endaddress
                mov     al,3
                call    us_voicecontrol
                dec     ecx
                jge     @@voiceloop
                mov     dx,[us_irqstat]
                in      al,dx
                mov     dx,[us_regselect]
                mov     al,49h
                out     dx,al
                add     dx,2
                in      al,dx
                mov     al,41h
                sub     dx,2
                out     dx,al
                add     dx,2
                in      al,dx
                sub     dx,2
                mov     al,4ch
                out     dx,al
                add     dx,2
                mov     al,07h
                out     dx,al
                popad
                clc
                sti
                ret
@@voices:       mov     ax,01h
                stc
                sti
                ret
@@no_ultrasound:mov     ax,02h
                stc
                sti
                ret

;͸͸
; Us_Probe                                                                  
;ľ                                                    
;                                                                            
; In:  -                                                                     
; Out: CF=1 if probe failed                                                  
;                                                                            
; Description: Probe ultrasound                                              
;                                                                            
;ľ
;  
                align   4
us_probe:       push    ax bx dx esi edi
                mov     dx,[us_regselect]
                mov     al,4ch
                out     dx,al
                add     dx,2
                sub     al,al
                out     dx,al
                call    us_delay
                call    us_delay
                sub     dx,2
                mov     al,4ch
                add     dx,2
                mov     al,01h
                out     dx,al
                call    us_delay
                call    us_delay
                sub     esi,esi
                call    us_peekdata
                mov     bl,al
                inc     esi
                call    us_peekdata
                mov     bh,al
                sub     edi,edi
                mov     al,055h
                call    us_pokedata
                inc     edi
                mov     al,0aah
                call    us_pokedata
                sub     esi,esi
                call    us_peekdata
                mov     dl,al
                inc     esi
                call    us_peekdata
                mov     dh,al
                sub     edi,edi
                mov     al,bl
                call    us_pokedata
                inc     edi
                mov     al,bh
                call    us_pokedata
                cmp     dl,055h
                jne     @@fail
                inc     esi
                cmp     dh,0aah
                jne     @@fail
                mov     dx,[us_base]
                add     dx,506h
                in      al,dx
                mov     [us_revisionid],al
                pop     edi esi dx bx ax
                clc
                ret
@@fail:         pop     edi esi dx bx ax
                stc
                ret

;͸͸
; Us_EnableOutput                                                           
;ľ                                                    
;                                                                            
; In:  -                                                                     
; Out: -                                                                     
;                                                                            
; Description: Enable output                                                 
;                                                                            
;ľ
;  
                align   4
us_enableoutput:push    dx ax
                mov     dx,[us_mixctrl]
                mov     al,[us_2x0]
                and     al,0fdh
                mov     [us_2x0],al
                out     dx,al
                pop     ax dx
                ret

;͸͸
; Us_DisableOutput                                                          
;ľ                                                    
;                                                                            
; In:  -                                                                     
; Out: -                                                                     
;                                                                            
; Description: Disable output                                                
;                                                                            
;ľ
;  
                align   4
us_disableoutput:
                push    dx ax
                mov     dx,[us_mixctrl]
                mov     al,[us_2x0]
                or      al,02h
                mov     [us_2x0],al
                out     dx,al
                pop     ax dx
                ret

;͸͸
; Us_SelectVoice                                                            
;ľ                                                    
;                                                                            
; In:  Al=voice number                                                       
; Out: -                                                                     
;                                                                            
; Description: Select active voice                                           
;                                                                            
;ľ
;  
                align   4
us_selectvoice: push    dx
                mov     dx,[us_page]
                dec     al
                out     dx,al
                inc     al
                pop     dx
                ret

;͸͸
; Us_FrequencyControl                                                       
;ľ                                                    
;                                                                            
; In:  EAx=Frequency                                                         
; Out: -                                                                     
;                                                                            
; Description: Set frequency for active voice                                
;                                                                            
;ľ
;  
                align   4
us_frequencycontrol:
                push    eax ebx ecx edx
                sub     ebx,ebx
                mov     bx,[us_divisor]
                mov     ecx,ebx
                shl     eax,9
                shr     ecx,1
                add     eax,ecx
                sub     edx,edx
                div     ebx
                add     eax,eax
                mov     bx,ax
                mov     dx,[us_regselect]
                mov     al,01h
                out     dx,al
                inc     dx
                mov     ax,bx
                out     dx,ax
                pop     edx ecx ebx eax
                ret

;͸͸
; Us_CurrentAddress                                                         
;ľ                                                    
;                                                                            
; In:  EAx=DRAM address                                                      
; Out: -                                                                     
;                                                                            
; Description: Set current address of active voice                           
;                                                                            
;ľ
;  
                align   4
us_currentaddress:
                push    eax ebx ecx edx
                mov     ebx,eax
                and     ebx,07fh
                shl     bx,9
                mov     ecx,eax
                shr     ecx,7
                and     cx,01fffh
                mov     dx,[us_regselect]
                mov     al,0bh
                out     dx,al
                mov     ax,bx
                inc     dx
                out     dx,ax
                mov     al,0ah
                dec     dx
                out     dx,al
                mov     ax,cx
                inc     dx
                out     dx,ax
                pop     edx ecx ebx eax
                ret

;͸͸
; Us_StartingAddress                                                        
;ľ                                                    
;                                                                            
; In:  EAx=DRAM address                                                      
; Out: -                                                                     
;                                                                            
; Description: Set loop starting address for active voice                    
;                                                                            
;ľ
;  
                align   4
us_startingaddress:
                push    eax ebx ecx edx
                mov     ebx,eax
                and     ebx,07fh
                shl     bx,9
                mov     ecx,eax
                shr     ecx,7
                and     cx,01fffh
                mov     dx,[us_regselect]
                mov     al,03h
                out     dx,al
                mov     ax,bx
                inc     dx
                out     dx,ax
                mov     al,02h
                dec     dx
                out     dx,al
                mov     ax,cx
                inc     dx
                out     dx,ax
                pop     edx ecx ebx eax
                ret

;͸͸
; Us_EndAddress                                                             
;ľ                                                    
;                                                                            
; In:  EAx=DRAM address                                                      
; Out: -                                                                     
;                                                                            
; Description: Set end address for active voice                              
;                                                                            
;ľ
;  
                align   4
us_endaddress:  push    eax ebx ecx edx
                mov     ebx,eax
                and     ebx,07fh
                shl     bx,9
                mov     ecx,eax
                shr     ecx,7
                and     cx,01fffh
                mov     dx,[us_regselect]
                mov     al,05h
                out     dx,al
                mov     ax,bx
                inc     dx
                out     dx,ax
                mov     al,04h
                dec     dx
                out     dx,al
                mov     ax,cx
                inc     dx
                out     dx,ax
                pop     edx ecx ebx eax
                ret

;͸͸
; Us_PanPosition                                                            
;ľ                                                    
;                                                                            
; In:  Al=Panning position (0..15)                                           
; Out: -                                                                     
;                                                                            
; Description: Set balance for active voice                                  
;                                                                            
;ľ
;  
                align   4
us_panposition: push    ax bx dx
                mov     bx,ax
                mov     dx,[us_regselect]
                mov     al,0ch
                out     dx,al
                add     dx,2
                mov     ax,bx
                out     dx,al
                pop     dx bx ax
                ret

;͸͸
; Us_CurrenVolume                                                           
;ľ                                                    
;                                                                            
; In:  Al - Volume                                                           
; Out: -                                                                     
;                                                                            
; Description: Set volume for active voice                                   
;                                                                            
;ľ
;  
                align   4
us_currentvolume:
                push    ax bx dx
                and     eax,0ffh
                mov     ax,[us_volumetable+eax*2]
                shl     ax,4
                mov     bx,ax
                mov     dx,[us_regselect]
                mov     al,09h
                out     dx,al
                inc     dx
                mov     ax,bx
                out     dx,ax
                call    us_delay
                out     dx,ax
                pop     dx bx ax
                ret

;͸͸
; Us_RampVolume                                                             
;ľ                                                    
;                                                                            
; In:  Al - End                                                              
; Out: -                                                                     
;                                                                            
; Description: Ramp current volume to destination volume                     
;                                                                            
;ľ
;  
                align   4
us_rampvolume:  push    ax bx cx dx
                and     eax,0ffh
                mov     bx,[us_volumetable+eax*2]
                shl     bx,4

                mov     dx,[us_regselect]

                mov     al,89h
                out     dx,al
                inc     dx
                in      ax,dx
                dec     dx
                mov     cx,ax

                mov     [@@mode],0
                cmp     cx,bx
                je      @@no_volchange
                jb      @@noswap
                xchg    cx,bx
                or      [@@mode],40h
@@noswap:       mov     cl,ch
                mov     bl,bh

                mov     al,06h
                out     dx,al
                add     dx,2
                mov     al,us_ramprate
                out     dx,al
                sub     dx,2

                mov     al,07h
                out     dx,al
                add     dx,2
                mov     al,cl
                out     dx,al
                sub     dx,2

                mov     al,08h
                out     dx,al
                add     dx,2
                mov     al,bl
                out     dx,al
                sub     dx,2

                mov     al,0dh
                out     dx,al
                add     dx,2
                mov     al,[@@mode]
                out     dx,al
                call    us_delay
                out     dx,al

@@no_volchange: pop     dx cx bx ax
                ret

                align   4
@@mode          db      ?



;͸͸
; Us_WaitVolRamp                                                            
;ľ                                            
;                                                                            
; In:  -                                                                     
;                                                                            
; Out: -                                                                     
;                                                                            
; Description: Wait until volumeramp is completed                            
;                                                                            
;ľ
;  
                align   4
us_waitvolramp: ife     us_fast
                push    ax cx dx
                mov     cx,[us_regselect]
@@wait:         mov     dx,cx
                mov     al,8dh
                out     dx,al
                add     dx,2
                in      al,dx
                test    al,1
                jz      @@wait
                pop     dx cx ax
                endif
                ret

;͸͸
; Us_VoiceControl                                                           
;ľ                                                    
;                                                                            
; In:  Al=VoiceControl byte  (check bits from GusSDK)                        
; Out: -                                                                     
;                                                                            
; Description: Set voice contol byte for active voice                        
;                                                                            
;ľ
;  
                align   4
us_voicecontrol:push    ax bx dx
                mov     bx,ax
                mov     dx,[us_regselect]
                sub     al,al
                out     dx,al
                add     dx,2
                mov     ax,bx
                out     dx,al
                pop     dx bx ax
                ret

